import 'package:example/pages/page_view_model.dart';
import 'package:example/pages/seconde_screen.dart';
import 'package:example/pages/tabs_demo.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pikachu_fac/widgets/loading_page.dart';

class HomePageWidget extends StatefulWidget {
  @override
  HomePageWidgetState createState() => new HomePageWidgetState();
}

class HomePageWidgetState extends State<HomePageWidget> {
  PageController _controller;

  Widget _buildTabWidget(BuildContext context) {
    const _pages = [const FirstTabWidget(), const SecondTabWidget(), const ThirdTabWidget()];
    return PageView.builder(
        physics: NeverScrollableScrollPhysics(), //viewPage禁止左右滑动
        controller: _controller,
        itemCount: _pages.length,
        itemBuilder: (context, index) => _pages[index]);
  }

  @override
  void initState() {
    _controller = PageController(initialPage: 0);
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        floatingActionButton: FloatingActionButton(
          tooltip: 'Increment',
          child: Icon(Icons.add),
          onPressed: () {
            switch (PageViewModel.get(context).pageModel.getState().currentTabIndex) {
              case 0:
                PageViewModel.get(context).loadFirstData(context);
                break;
              case 1:
                PageViewModel.get(context).pageModel.setState((value, _) {
                  _.state.count++;
                  print('click ${_.state}');
                });

                break;
              case 2:
                PageViewModel.get(context).loadThirdData(context,
                    instance: (PageViewModel.get(context).thirdPageModel.getState().index == 0 ? 'HOME' : 'APPAREL'));
                break;
            }
          },
        ),
        body: _buildTabWidget(
          context,
        ),
        bottomNavigationBar: BottomNavigationBar(
          currentIndex: (PageViewModel.get(context).pageModel.getState().currentTabIndex),
          onTap: (index) {
            _controller.jumpToPage(index);
            PageViewModel.get(context).pageModel.setState((value, _) {
              _.state.currentTabIndex = index;
            });
          },
          items: <BottomNavigationBarItem>[
            BottomNavigationBarItem(title: Text('首页'), icon: Icon(Icons.home)),
            BottomNavigationBarItem(title: Text('书籍'), icon: Icon(Icons.book)),
            BottomNavigationBarItem(title: Text('我的'), icon: Icon(Icons.perm_identity)),
          ],
        ));
  }
}

class FirstTabWidget extends StatefulWidget {
  @override
  FirstTabWidgetState createState() => new FirstTabWidgetState();

  const FirstTabWidget({Key key}) : super(key: key);
}

class FirstTabWidgetState extends State<FirstTabWidget> {
  @override
  void didChangeDependencies() {
    PageViewModel.get(context).loadFirstData(context);
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      appBar: AppBar(
        title: Text("LoadingPage"),
      ),
      body: _buildFirstContentWidget(context),
    );
  }
}

LoadingPageWidget _buildFirstContentWidget(BuildContext context) {
  return LoadingPageWidget(
    failureWidget: Container(
      alignment: Alignment.center,
      child: Center(
        child: Text(
          'custom fail hint,click retry',
          style: TextStyle(
            fontSize: 30,
            color: Colors.red,
          ),
        ),
      ),
    ),
    emptyWidget: Container(
      alignment: Alignment.center,
      child: Center(
        child: Text(
          "Custom empty hint",
          style: TextStyle(
            fontSize: 30,
            color: Colors.blue,
          ),
        ),
      ),
    ),
    childBuilder: (value, context) {
      return PageViewModel.get(context).pageModel.buildWithObserve((ctx, _state) {
        return Container(
          alignment: Alignment.center,
          child: Center(
            child: Text(
              _state.dataFromNetwork,
              style: TextStyle(
                fontSize: 30,
                color: Colors.blue,
              ),
            ),
          ),
        );
      }, memo: (state) => [state.dataFromNetwork]);
    },
    onRetryClick: () {
      PageViewModel.get(context).loadFirstData(context);
    },
  );
}
